/* ----------------------------- MNI Header -----------------------------------
@NAME       : itkMincImageIOFactory.h
@DESCRIPTION: 
@COPYRIGHT  :
              Copyright 2012 Vladimir Fonov, McConnell Brain Imaging Centre, 
              Montreal Neurological Institute, McGill University.
              Permission to use, copy, modify, and distribute this
              software and its documentation for any purpose and without
              fee is hereby granted, provided that the above copyright
              notice appear in all copies.  The author and McGill University
              make no representations about the suitability of this
              software for any purpose.  It is provided "as is" without
              express or implied warranty.
---------------------------------------------------------------------------- */

#ifndef __itkMincImageIOFactory_h
#define __itkMincImageIOFactory_h

#ifdef _MSC_VER
#pragma warning ( disable : 4786 )
#endif

#include "itkObjectFactoryBase.h"
#include "itkImageIOBase.h"

namespace itk
{
/** \class MincImageIOFactory
  * \ingroup  ITKIOMINC
  * \brief Create instances of MincImageIO objects using an object factory.
  */
class ITK_EXPORT MincImageIOFactory : public ObjectFactoryBase
{
public:
  /** Standard class typedefs. */
  typedef MincImageIOFactory   Self;
  typedef ObjectFactoryBase  Superclass;
  typedef SmartPointer<Self>  Pointer;
  typedef SmartPointer<const Self>  ConstPointer;

  /** Class methods used to interface with the registered factories. */
  virtual const char* GetITKSourceVersion(void) const;
  virtual const char* GetDescription(void) const;

  /** Method for class instantiation. */
  itkFactorylessNewMacro(Self);

  /** Run-time type information (and related methods). */
  itkTypeMacro(MincImageIOFactory, ObjectFactoryBase);

  /** Register one factory of this type  */
  static void RegisterOneFactory(void)
  {
    MincImageIOFactory::Pointer metaFactory = MincImageIOFactory::New();
    
#if ( ITK_VERSION_MAJOR > 3 )
    //try to be higher priority then built-in HDF5 IO class
    ObjectFactoryBase::RegisterFactory(metaFactory, ObjectFactoryBase::INSERT_AT_FRONT );
#else
    ObjectFactoryBase::RegisterFactory(metaFactory);
#endif 
  }

protected:
  MincImageIOFactory();
  ~MincImageIOFactory();
  virtual void PrintSelf(std::ostream& os, Indent indent) const;

private:
  MincImageIOFactory(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
};

void RegisterMincIO(void);

} // end namespace itk

#endif
